"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSvelteTypeScriptReader = void 0;
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const compiler_1 = require("svelte/compiler");
/**
 * Returns a reader that returns a virtual TypeScript file
 * named foo.svelte.ts for every corresponding foo.svelte file.
 */
function createSvelteTypeScriptReader(reader) {
    return new SvelteTypeScriptReader(reader);
}
exports.createSvelteTypeScriptReader = createSvelteTypeScriptReader;
class SvelteTypeScriptReader {
    constructor(reader) {
        var _a;
        this.reader = reader;
        this.listeners = new vfs_1.ReaderListeners();
        reader.listeners.add({
            onChange: (absoluteFilePath, info) => {
                this.listeners.notify(absoluteFilePath, info);
            },
        });
        this.observe = (_a = reader.observe) === null || _a === void 0 ? void 0 : _a.bind(reader);
    }
    async read(filePath) {
        if (filePath.endsWith(".svelte.ts")) {
            const source = await this.reader.read(filePath.substr(0, filePath.length - 3));
            if ((source === null || source === void 0 ? void 0 : source.kind) !== "file") {
                return null;
            }
            return {
                kind: "file",
                name: path_1.default.basename(filePath),
                realPath: async () => null,
                lastModifiedMillis: () => source.lastModifiedMillis(),
                read: async () => convertToTypeScript(await source.read()),
                size: () => source.size(),
            };
        }
        const entry = await this.reader.read(filePath);
        if ((entry === null || entry === void 0 ? void 0 : entry.kind) === "directory") {
            return {
                ...entry,
                entries: async () => {
                    const realEntries = await entry.entries();
                    return Promise.all([
                        ...realEntries.map((e) => this.read(path_1.default.join(filePath, e.name))),
                        ...realEntries
                            .filter((e) => e.name.endsWith(".svelte"))
                            .map((e) => this.read(path_1.default.join(filePath, e.name + ".ts"))),
                    ]).then((entries) => entries.filter(Boolean));
                },
            };
        }
        return entry;
    }
    readSync(filePath) {
        if (filePath.endsWith(".svelte.ts")) {
            const source = this.reader.readSync(filePath.substr(0, filePath.length - 3));
            if ((source === null || source === void 0 ? void 0 : source.kind) !== "file") {
                return null;
            }
            return {
                kind: "file",
                name: path_1.default.basename(filePath),
                realPath: () => null,
                lastModifiedMillis: () => source.lastModifiedMillis(),
                read: () => convertToTypeScript(source.read()),
                size: () => source.size(),
            };
        }
        const entry = this.reader.readSync(filePath);
        if ((entry === null || entry === void 0 ? void 0 : entry.kind) === "directory") {
            return {
                ...entry,
                entries: () => {
                    const realEntries = entry.entries();
                    return [
                        ...realEntries.map((e) => this.readSync(path_1.default.join(filePath, e.name))),
                        ...realEntries
                            .filter((e) => e.name.endsWith(".svelte"))
                            .map((e) => this.readSync(path_1.default.join(filePath, e.name + ".ts"))),
                    ].filter(Boolean);
                },
            };
        }
        return entry;
    }
}
function convertToTypeScript(svelteTemplateSource) {
    // Note: preprocess returns a promise, but it runs immediately.
    let scriptContent = "";
    (0, compiler_1.preprocess)(svelteTemplateSource, [
        {
            script: ({ content }) => {
                scriptContent = content;
                return {
                    code: "",
                };
            },
        },
    ]);
    return scriptContent;
}
